<?php

namespace ApiQL\Language;

/**
 * trait JsonField - trait for handling json fields
 *
 * @package ApiQL\Language
 */
trait JsonField
{

    /**
     * Get json property value
     *
     * @param mixed $value - field value
     *
     * @return string
     */
    private function getJsonPropertyValue($value)
    {
        if (is_array($value) && array_key_first($value) !== 0) {
            $value = $value[array_key_first($value)];
        }
        return $value;
    }

    /**
     * Get json field name
     *
     * @param string $field - field
     * @param mixed $value - field value
     *
     * @return string
     */
    private function getJsonFieldName(string $field, $value)
    {
        if (strpos($field, '[') !== false) {
            $field = str_replace(['[', ']'], '_', $field);
        } elseif (is_array($value) && array_key_first($value) !== 0) {
            $field = $field . '_' . array_key_first($value) . '_';
        }
        return $field;
    }


    /**
     * Get json field name
     *
     * @param string $field - field
     * @param mixed $value - field value
     *
     * @return string
     */
    private function getJsonFieldValue(string $field, $value = null, bool $isArray = false)
    {
        if (strpos($field, '[') !== false) {
            preg_match('/(.+?)\[/is', $field, $fieldName);
            preg_match('/\[(.+?)\]/is', $field, $fieldProp);
            $field = "\"getPropertyById\"($fieldName[1], '$fieldProp[1]')";
            if (!is_null($value)) {
                if ($isArray && is_string($value)) {
                    $value = explode(",", $value);
                }
                $fieldType = $this->getJsonFieldType($value);
                if (!empty($fieldType)) {
                    $field .= "::$fieldType";
                }
            }
        } elseif (!is_null($value) && is_array($value) && array_key_first($value) !== 0) {
            $fieldProp = array_key_first($value);
            $field = "\"getPropertyById\"($field, '$fieldProp')";
            if ($isArray && is_string($value[$fieldProp])) {
                $value[$fieldProp] = explode(",", $value[$fieldProp]);
            }
            $fieldType = $this->getJsonFieldType($value[$fieldProp]);
            if (!empty($fieldType)) {
                $field .= "::$fieldType";
            }
        }
        return $field;
    }

    /**
     * Get json field type
     *
     * @param mixed $value - field value
     *
     * @return string
     */
    private function getJsonFieldType($value)
    {
        if ($this->isBoolean($value)) {
            return "boolean";
        }
        if ($this->isInt($value)) {
            return "int";
        }
        if ($this->isFloat($value)) {
            return "float";
        }
        return '';
    }

    /**
     * Check value is boolean
     *
     * @param mixed $value - field value
     *
     * @return bool
     */
    private function isBoolean($value): bool
    {
        return is_bool($value) || $value == "true" || $value == "false";
    }

    /**
     * Check value is integer
     *
     * @param mixed $value - field value
     *
     * @return bool
     */
    private function isInt($value): bool
    {
        if (is_array($value)) {
            return $this->isInt($value[0]);
        }
        return is_int($value) || ($value === (string) (int) $value);
    }

    /**
     * Check value is float
     *
     * @param mixed $value - field value
     *
     * @return bool
     */
    private function isFloat($value): bool
    {
        if (is_array($value)) {
            return $this->isFloat($value[0]);
        }
        return is_float($value) || ($value === (string) (float) $value);
    }
}
