<?php

namespace ApiQL\Language;

use ApiQL\Language\Interpreters\{
    AndInterpreter,
    OrInterpreter,
    EqualsInterpreter,
    NotEqualsInterpreter,
    LessThanInterpreter,
    LessThanOrEqualInterpreter,
    GreaterThanInterpreter,
    GreaterThanOrEqualInterpreter,
    IsNullInterpreter,
    IsNotNullInterpreter,
    LikeInterpreter,
    NotLikeInterpreter,
    InInterpreter,
    NotInInterpreter,
    SpecInterpreter
};

/**
 * Class LanguageFactory
 *
 * @package ApiQL\Language
 */
class LanguageFactory
{
    /**
     * Build interpreter based on provided data
     *
     * @param array $data - expression data, used to build interpreter
     * @param string $operator - default operator
     *
     * @return InterpreterInterface
     *
     * @throws InvalidArgumentException
     */
    public static function build(array $data, string $operator = null)
    {
        if (!is_null($operator)) {
            $data_ = $data;
        } else {
            $operator = array_key_first($data);
            $data_ = $data[$operator];
        }
        switch ($operator) {
            case 'and':
                return new AndInterpreter($data_);
            case 'or':
                return new OrInterpreter($data_);
            case 'eq':
                return new EqualsInterpreter($data_);
            case 'neq':
                return new NotEqualsInterpreter($data_);
            case 'lt':
                return new LessThanInterpreter($data_);
            case 'lte':
                return new LessThanOrEqualInterpreter($data_);
            case 'gt':
                return new GreaterThanInterpreter($data_);
            case 'gte':
                return new GreaterThanOrEqualInterpreter($data_);
            case 'is_null':
                return new IsNullInterpreter($data_);
            case 'is_not_null':
                return new IsNotNullInterpreter($data_);
            case 'like':
                return new LikeInterpreter($data_);
            case 'not_like':
                return new NotLikeInterpreter($data_);
            case 'in':
                return new InInterpreter($data_);
            case 'not_in':
                return new NotInInterpreter($data_);
            case 'spec':
                return new SpecInterpreter($data);
            default:
                throw new InvalidArgumentException('Interpreter not found');
        }
    }
}
